using System;
using System.Collections.Generic;
using System.Text;
using System.Data;

namespace ConstraintsForeignKey
{
    class Program
    {
        static void PrintTable(DataTable t)
        {
            Console.WriteLine("Tabela: {0}", t.TableName);
            foreach (DataRow r in t.Rows)
            {
                foreach (DataColumn c in t.Columns)
                {
                    if (r[c].GetType()==Type.GetType("System.Int32"))
                        Console.Write("{0,3}", r[c]);
                    else
                        Console.Write("{0,12}", r[c]);

                }
                Console.WriteLine();
            }
        }

        static void Main(string[] args)
        {
            DataSet ds = new DataSet();
            ds.EnforceConstraints = true;

            DataTable dtJednostka = new DataTable("Jednostka");
            dtJednostka.Columns.Add(new DataColumn("ID", Type.GetType("System.Int32")));
            dtJednostka.Columns.Add(new DataColumn("Nazwa", Type.GetType("System.String")));
            dtJednostka.Columns.Add(new DataColumn("Kod", Type.GetType("System.String")));
            dtJednostka.Constraints.Add(new UniqueConstraint("PK_Jednostka",
                new DataColumn[] { dtJednostka.Columns["ID"] }, true));
            dtJednostka.Constraints.Add(new UniqueConstraint("U_Jednostka_NazwaKod",
                new DataColumn[] { dtJednostka.Columns["Nazwa"], dtJednostka.Columns["Kod"] }, false));

            DataTable dtOsoba = new DataTable("Osoba");
            dtOsoba.Columns.Add(new DataColumn("ID", Type.GetType("System.Int32")));
            dtOsoba.Columns.Add(new DataColumn("Jednostka_ID", Type.GetType("System.Int32")));
            dtOsoba.Columns.Add(new DataColumn("Jednostka_Nazwa", Type.GetType("System.String")));
            dtOsoba.Columns.Add(new DataColumn("Jednostka_Kod", Type.GetType("System.String")));
            dtOsoba.Columns.Add(new DataColumn("Imie", Type.GetType("System.String")));
            dtOsoba.Columns.Add(new DataColumn("Nazwisko", Type.GetType("System.String")));
            dtOsoba.Columns.Add(new DataColumn("Telefon", Type.GetType("System.String")));
            dtOsoba.PrimaryKey = new DataColumn[] { dtOsoba.Columns["ID"] };

            ForeignKeyConstraint fkJednostkaID = new ForeignKeyConstraint(
                "FK_Jednostka_ID",
                dtJednostka.Columns["ID"],
                dtOsoba.Columns["Jednostka_ID"]);
            fkJednostkaID.AcceptRejectRule = AcceptRejectRule.Cascade;
            // Za drugim razem komentujemy ponisze
            fkJednostkaID.DeleteRule = Rule.SetNull;
            fkJednostkaID.UpdateRule = Rule.Cascade;
            dtOsoba.Constraints.Add(fkJednostkaID);

            ForeignKeyConstraint fkJednostkaNazwaKod = new ForeignKeyConstraint(
                "FK_Jednostka_NazwaKod",
                new DataColumn[] { dtJednostka.Columns["Nazwa"], dtJednostka.Columns["Kod"] },
                new DataColumn[] { dtOsoba.Columns["Jednostka_Nazwa"], dtOsoba.Columns["Jednostka_Kod"] });
            fkJednostkaNazwaKod.AcceptRejectRule = AcceptRejectRule.Cascade;
            // Za drugim razem komentujemy ponisze
            fkJednostkaNazwaKod.DeleteRule = Rule.SetNull;
            fkJednostkaNazwaKod.UpdateRule = Rule.Cascade;
            dtOsoba.Constraints.Add(fkJednostkaNazwaKod);

            ds.Tables.Add(dtJednostka);
            ds.Tables.Add(dtOsoba);

            dtJednostka.Rows.Add(1, "Wrocaw", "71");
            dtJednostka.Rows.Add(2, "Warszawa", "22");
            dtJednostka.Rows.Add(3, "Krakw", "12");

            dtOsoba.Rows.Add(1, 1, "Wrocaw", "71", "Jan01", "Kowalski01", "071-345-14-14");
            dtOsoba.Rows.Add(2, 1, "Wrocaw", "71", "Jan02", "Kowalski02", "071-345-14-15");
            dtOsoba.Rows.Add(3, 2, "Warszawa", "22", "Jan03", "Kowalski03", "022-345-14-16");
            dtOsoba.Rows.Add(4, 2, "Warszawa", "22", "Jan04", "Kowalski04", "022-345-14-17");
            dtOsoba.Rows.Add(5, 3, "Krakw", "12", "Jan05", "Kowalski05", "012-345-14-18");
            dtOsoba.Rows.Add(6, 3, "Krakw", "12", "Jan06", "Kowalski06", "012-345-14-19");

            Console.WriteLine("PRZED ZMIANAMI");
            PrintTable(dtJednostka);
            PrintTable(dtOsoba);

            dtJednostka.Rows[0]["ID"] = 4;
            dtJednostka.Rows[1]["Nazwa"] = "WARSZAWA";
            dtJednostka.Rows[1]["Kod"] = "022"; // bez tej zmiany nie bdzie Cascade!
            dtJednostka.Rows[2].Delete();

            //dtJednostka.Rows[0].AcceptChanges();
            //dtJednostka.AcceptChanges();
            //dtOsoba.AcceptChanges();

            Console.WriteLine("PO ZMIANACH");
            PrintTable(dtJednostka);
            PrintTable(dtOsoba);

            Console.ReadLine();
        }
    }
}
